/// <reference path="../typings/index.d.ts" />

const assert: any = require('assert');

import {Condvar} from '../lib/condvar';
import {Mutex} from '../lib/mutex';

describe('Condvar', function () {
    it('should be notifiable with no waiters', function () {
        let mu = new Mutex();
        let cond = new Condvar(mu);
        cond.notify();
    });

    it('should be all notifiable with no waiters', function () {
        let mu = new Mutex();
        let cond = new Condvar(mu);
        cond.notifyAll();
    });

    it('should notify one waiter', function (done) {
        let mu = new Mutex();
        assert(mu.tryLock());
        let cond = new Condvar(mu);
        cond.wait().then(() => done());
        cond.notify();
    });

    it('should notify multiple times', function (done) {
        let mu = new Mutex();
        assert(mu.tryLock());
        let cond = new Condvar(mu);
        cond.wait().then(() => {
            mu.unlock();

            assert(mu.tryLock());
            cond.wait().then(() => done());
            cond.notify();
        });
        cond.notify();
    });

    it('should notify first waiter', function (done) {
        let mu = new Mutex();
        let cond = new Condvar(mu);

        assert(mu.tryLock());
        cond.wait().then(() => done());

        assert(mu.tryLock());
        cond.wait().then(() => assert.ok(false, 'wrong waiter woken up'));

        cond.notify();
    });

    it('should all notify all waiters', function (done) {
        let n = 3;

        function countdown() {
            --n;
            if (n === 0) {
                done();
            }
        }

        let mu = new Mutex();
        let cond = new Condvar(mu);

        assert(mu.tryLock());
        cond.wait().then(() => {
            countdown();
            mu.unlock();
        });

        assert(mu.tryLock());
        cond.wait().then(() => {
            countdown();
            mu.unlock();
        });

        assert(mu.tryLock());
        cond.wait().then(() => {
            countdown();
            mu.unlock();
        });

        cond.notifyAll();
    });
});
