/// <reference path="../typings/index.d.ts" />

const assert: any = require('assert');

import {Mutex} from '../lib/mutex';

describe('Mutex', function () {
    it('should be lockable when unlocked', function () {
        let mu = new Mutex();
        assert(mu.tryLock());
    });

    it('should not be lockable when locked', function () {
        let mu = new Mutex();
        assert(mu.tryLock());
        assert(!mu.tryLock());
    });

    it('should not be unlockable when unlocked', function () {
        let mu = new Mutex();
        assert.throws(() => mu.unlock());
    });

    it('should be lockable multiple times', function () {
        let mu = new Mutex();
        assert(mu.tryLock());
        mu.unlock();
        assert(mu.tryLock());
        mu.unlock();
    });

    it('should lock immediately when unlocked', function (done) {
        let mu = new Mutex();
        mu.lock().then(() => done());
    })

    it('should run callbacks when unlocked', function (done) {
        let mu = new Mutex();
        mu.lock();
        mu.lock().then(() => done());
        mu.unlock();
    })

    it('should run callbacks in order', function (done) {
        let n = 3;
        let mu = new Mutex();

        function countdown() {
            --n;
            if (n === 0) {
                done();
            }
        }

        mu.lock().then(() => {
            countdown();
        });

        mu.lock().then(() => {
            countdown();
        });

        mu.lock().then(() => {
            countdown();
        });

        mu.unlock();
        mu.unlock();
        mu.unlock();
    });
});
