/// <reference path="../typings/index.d.ts" />

const assert: any = require('assert');

import {Semaphore} from '../lib/semaphore';

describe('Semaphore', function () {
    it('should be waitable when positive', function () {
        let sem = new Semaphore(1);
        assert(sem.tryWait());
    });

    it('should not be waitable when zero', function () {
        let sem = new Semaphore(0);
        assert(!sem.tryWait());
    });

    it('should not be waitable when negative', function () {
        let sem = new Semaphore(-1);
        assert(!sem.tryWait());
    });

    it('should be waitable after signal', function () {
        let sem = new Semaphore(0);
        sem.signal();
        assert.strictEqual(sem.value, 1);
        assert(sem.tryWait());
    });

    it('should be waitable multiple times', function () {
        let sem = new Semaphore(0);
        sem.signal();
        assert(sem.tryWait());
        sem.signal();
        assert(sem.tryWait());
    });

    it('should not be waitable after wait', function () {
        let sem = new Semaphore(1);
        assert(sem.tryWait());
        assert(!sem.tryWait());
    });

    it('should run callbacks when signaled', function (done) {
        let sem = new Semaphore(1);
        sem.wait().then(() => done());
        sem.signal();
    });

    it('should run callbacks in order when signaled', function (done) {
        let n = 2;

        function countdown() {
            --n;
            if (n === 0) {
                done();
            }
        }

        let sem = new Semaphore(0);
        sem.wait().then(() => countdown());
        sem.wait().then(() => countdown());
        sem.signal();
        sem.signal();
    });
});
